import os
import json

class LocalStore:
    def __init__(self, path='app.json') -> None:
        self.json_path = path

    def _load_json(self):
        if not os.path.exists(self.json_path):
            return {}
        try:
            with open(self.json_path, 'r', encoding="utf-8") as json_file:
                return json.load(json_file)
        except json.JSONDecodeError:
            return {}

    def _save_json(self, data):
        try:
            dir_path = os.path.dirname(self.json_path)
            if dir_path and not os.path.exists(dir_path):
                os.makedirs(dir_path)
            with open(self.json_path, 'w', encoding="utf-8") as f:
                json.dump(data, f, ensure_ascii=False, indent=4)
            return True
        except IOError as e:
            print(f"Error saving data: {e}")
            return False

    # 获取全部数据
    def getAll(self):
        return self._load_json()

    # 获取单个数据
    def get(self, key=None):
        data = self._load_json()
        if key is None:
            return data
        return data.get(key, None)

    # 删除数据
    def delete(self, key):
        data = self._load_json()
        if key in data:
            data.pop(key)
            return self._save_json(data)
        else:
            print(f"No data found for key: {key}")
            return False

    # 保存数据
    def save(self, key, value=None):
        data = self._load_json()
        
        if isinstance(key, str):
            data[key] = value
        elif isinstance(key, dict):
            data.update(key)
        elif isinstance(key, list):
            data = key
        
        return self._save_json(data)


#!/usr/bin/env python3
# -*- coding:utf-8-*-
# 全局变量管理模块global_variable

def _init():
    """在主模块初始化"""
    global IS_INIT
    IS_INIT = False
    
    global GLOBALS_DICT
    GLOBALS_DICT = {}

def set(name, value):
    """设置"""
    try:
        GLOBALS_DICT[name] = value
        return True
    except KeyError:
        return False

def get(name, default=None):
    """取值"""
    if name in GLOBALS_DICT:
        return GLOBALS_DICT[name]
    else:
        return default

def printParam(name):
    """打印参数"""
    if not name:
        import json
        print('GLOBALS_DICT:', json.dumps(GLOBALS_DICT, indent=2, ensure_ascii=False))
        return
    if name in GLOBALS_DICT:
        print(name+':', get(name))
    else:
        print('Not Found')

def getDict():
    """取值"""
    try:
        return GLOBALS_DICT
    except KeyError:
        return "Not Found"


from .Store import Store
from .Page import Page
from .Param import Param
from .Children import Children
from .Callback import Callback
from .LocalStore import LocalStore

__all__ = [
  "Store",
  "Page",
  "PageManager",
  "Param",
  "Children",
  "Callback",
  "LocalStore"
]


from . import global_variable as glv

class Store:
  def __init__(self, dict=None):
    if dict:
      glv._init()
      glv.GLOBALS_DICT = dict

  def set(self, key, value):
    """设置值"""
    if isinstance(key, str):
      glv.set(key, value)
    elif isinstance(key, dict):
      for each in key.keys():
        glv.set(each, key[each])
    else:
      raise TypeError("key must be str or dict")

  def setDict(self, setDict:dict):
    """设置字典"""
    if isinstance(setDict, dict):
      glv.GLOBALS_DICT = setDict
      
  def get(self, key:str, default=None):
    """获取值"""
    return glv.get(key, default)
  
  def has(self, key:str):
    """判断是否存在"""
    return key in glv.GLOBALS_DICT
  
  def getDict(self):
    """获取字典"""
    return glv.GLOBALS_DICT
  
  def print(self, key:str):
    """打印参数"""
    return glv.printParam(key)
  
  def keys(self):
    """获取所有键"""
    return list(glv.GLOBALS_DICT.keys())
  
  def values(self):
    """获取所有值"""
    return list(glv.GLOBALS_DICT.values())
  
  def remove(self, key=None):
    """移除指定值或所有值"""
    if not key:
      glv.GLOBALS_DICT = {}
    else:
      del glv.GLOBALS_DICT[key]


import os
from .LocalStore import LocalStore

# 高效参数存取模块
class Param(object):
  def __init__(self, filePath=None, default={}):
    self.filePath = filePath
    self.localStore = LocalStore(filePath) if filePath  else None
    self.count = 0
    self.default = default
    self.load()


  # 加载磁盘数据到内存
  def load(self):
    data = self.localStore.getAll() if self.localStore else {}
    if not data:
      self.count += 1
      self.data = self.default
    else:
      self.data = data
    return self.data


  # 判断参数中是否有数据
  def has(self, key:str):
    return key in self.data
  
  # 获取内存中的数据
  def get(self, key=None, default=None):
    # 如果没有指令，返回全部信息
    if not key and default == None:
      return self.data
    
    # 输入指令转化为参数列表
    attrList = key.split("/")
    # 参数列表倒置，从浅入深访问数据
    attrList.reverse()
    def extract(data):
      try:
        # 获取一层参数
        attr = attrList.pop(-1)
        # 尝试解释参数是否是数字，是就改为数字，不是就算了
        try:
          attr = int(attr)
        except:
          pass
        # 获取这层参数的值
        value = data[attr]
        # 如果参数列表还有参数，继续提取值，没有参数返回当前的值
        if len(attrList)>0:
          return extract(value)
        else:
          return value
      except:
        return default
    return extract(self.data)
    

  # 更新内存中的数据
  def update(self, key, value=None):
    self.count += 1
    if isinstance(key, list) or isinstance(key, dict):
      self.data = key
      return
    # 输入指令转化为参数列表
    attrList = key.split("/")
    # 参数列表倒置，从浅入深访问数据
    attrList.reverse()
    # 递归访问，直到找到输入指令所在位置的参数，并进行更新返回True，否则返回False
    def updateVal(data):
      try:
        # 获取一层参数
        attr = attrList.pop(-1)
        
        # 尝试解释参数是否是数字，是就改为数字，不是就算了
        isNum = False
        try:
          attr = int(attr)
          isNum = True
        except:
          pass

        # 如果参数列表还有参数，继续提取值，没有参数返回当前的值
        if len(attrList)>0:
          return updateVal(data[attr])
        # 如果参数列表没有参数了，赋值就好，但列表要存在能赋值
        else:
          # 如果参数是数字，判断数据是否是列表
          if isNum:
            if isinstance(data[attr], list) and len(data[attr])>= attr+1:
              data[attr] = value
              return True, "SUCCESS"
            else:
              return False, "NOT LIST"
          # 如果参数不是数字，直接赋值
          else:
            data[attr] = value
            return True, "SUCCESS"
      except:
        return False, "UPDATE FAIL"
    return updateVal(self.data)


  # 设置内存中的数据
  def set(self, key, value=None):
    self.count += 1
    if isinstance(key, list) or isinstance(key, dict):
      self.data = key
      return
    # 输入指令转化为参数列表
    attrList = key.split("/")
    # 参数列表倒置，从浅入深访问数据
    attrList.reverse()
    # 递归访问，直到找到输入指令所在位置的参数，并进行更新返回True，否则返回False
    def setVal(data):
      # 获取一层参数
      attr = attrList.pop(-1)

      # 如果参数能访问到数据，一直访问到最后一层，然后根据最后一层是什么数据执行相应的操作
      try:
        # 尝试解释参数是否是数字，是就改为数字，不是就算了
        isNum = False
        try:
          attr = int(attr)
          isNum = True
        except:
          pass

        # 如果参数列表还有参数，继续提取值，没有参数返回当前的值
        if len(attrList)>0:
          return setVal(data[attr])
        # 如果参数列表没有参数了，赋值就好，但列表要存在能赋值
        else:
          # 如果参数是数字，判断数据是否是列表
          if isNum:
            # 如果当前数据是列表，并且指定位置已存在数据，直接更换数据
            if isinstance(data, list) and len(data)>= attr+1:
              data[attr] = value
              return True, "SUCCESS"
            # 如果当前数据是列表，并且指定位置没有数据，保留原数据，分隔位置添加None子元素，指定位置放置指定数据
            elif isinstance(data, list) and len(data) < attr+1:
              for i in range(attr):
                try:
                  old = data[i]
                except:
                  old = None
                data[i] = old
              data[attr] = value
              return True, "SUCCESS"
            # 如果当前数据不是列表，创建一个空列表，在指定位置放置指定数据，其他位置设置为None
            else:
              return False, "NOT LIST"
          # 如果参数不是数字，直接赋值
          else:
            data[attr] = value
            return True, "SUCCESS"
      except:
        # 当前参数是数字
        isNum = False
        try:
          attr = int(attr)
          isNum = True
        except:
          pass
        # 如果参数列表还有值
        if len(attrList)>0:
          # 下一个参数是数字
          next_attr = attrList[-1] 
          nextIsNum = False
          try:
            next_attr = int(next_attr)
            nextIsNum = True
          except:
            pass
          
          _create = [] if nextIsNum else {}
          if isNum:
            # 如果data不是列表，则无法添加元素
            try:
              for i in range(attr):
                try:
                  data[i]
                except:
                  data.append(None)
              data.append(_create)
            except:
              return False, "NOT LIST"
          else:
            # 如果data不是对象，则无法添加元素
            try:
              data[attr] = _create
            except:
              return False, "NOT OBJECT"
          # 继续深挖
          if len(attrList)>0:
            return setVal(_create)
          return True, "SUCCESS"
        # 如果参数列表没有值了
        else:
          # 将指定值，赋值给当前参数
          if isNum:
            # 如果data不是列表，则无法添加元素
            try:
              for i in range(attr):
                try:
                  data[i]
                except:
                  data.append(None)
              data.append(value)
            except:
              return False, "NOT LIST"
          else:
            # 如果data不是对象，则无法添加元素
            try:
              data[attr] = value
            except:
              return False, "NOT OBJECT"
          return True, "SUCCESS"
    return setVal(self.data)
  

  # 保存内存的数据到磁盘
  def save(self, key=None, value=None):
    if isinstance(key, str):
      self.set(key,value)
      self.count += 1
    elif isinstance(key, list) or isinstance(key, dict):
      self.data = key
      self.count += 1
    
    if self.count > 0:
      # print(f">>>{self.filePath} save data:", self.data)
      self.localStore.save(self.data)
      self.count = 0
    else:
      pass
      # print(f">>>{self.filePath} save 数据未更改，无需保存")

  
  # 删除数据
  def delete(self, key, isSave=True):
    self.set(key, None)
    if isSave:
      self.save()
    

  # 读取数据并拼接路径
  def pathJoin(self, key, *args):
    return os.path.join(self.get(key), *args).replace("\\","/")
  
  
  # 清空内存中的数据
  def clear(self):
    self.data = None


  def child(self,path=None, default=None):
    return Param(path, default)
  

  def walk(self, pick=None, check=None, type="all", path=""):
    def func_all(data:list|dict, path:str, pick=None):
      if isinstance(data, list):
        for i in range(len(data)):
          pathList = [] if path == "" else path.split("/")
          pathList.append(str(i))
          func_all(data[i], "/".join(pathList), pick)
      elif isinstance(data, dict):
        check and check(path, data)
        for key in data.keys():
          pathList = [] if path == "" else path.split("/")
          pathList.append(key)
          func_all(data[key], "/".join(pathList), pick)
      else:
        pick and pick(path, data)

    def func_dict(data:dict, path:str, pick=None):
      if isinstance(data, dict):
        for key in data.keys():
          pathList = [] if path == "" else path.split("/")
          pathList.append(key)
          func_dict(data[key], "/".join(pathList), pick)
      else:
        pick and pick(path, data)
    
    if type == "all":
      func_all(data=self.data, path=path, pick=pick)
    elif type == "dict":
      func_dict(data=self.data, path=path, pick=pick)
    else:
      print("错误的类型")


import os
from .Param import Param

class Children(object):
  def __init__(self):
    self.components = dict()
    self.props:Param
  # 查看当前页面的子页面信息
  def info(self, level="——"):
    info_string = ""
    for key in self.components.keys():
      info_string += level + key + "\n"
      info_string += self.components.get(key).children.info(level="  "+level)
    return info_string

  # 子页面初始化
  def setup(self, props=None, createPage=None):
    loadProps(self, props)
    # 初始化存在的页面
    keys =  list(self.components.keys())
    for key in keys:
      component = self.components[key]
      if hasattr(component, 'setup'):
        if self.props.has(component.name):
          component.setup(self.props.get(component.name))
        else:
          component.setup()
    # 初始化缺失的页面    
    props = self.props.get()
    children = props.keys()
    for child in children:
      if child not in self.components.keys():
        component = createPage(child)
        self.components[child] = component
        component.setup(self.props.get(child))

  # 添加子页面
  def add(self, name:str, component:object):
    if name not in self.components.keys():
      self.components[name] = component
    else:
      AttributeError(name=f"{name}组件已存在")

  # 获取子页面
  def get(self, name:str):
    if name in self.components.keys():
      return self.components[name]
    else:
      return None
    
  def has(self, name:str):
    return name in self.components.keys()
  
  # 显示组件
  def show(self, name):
    if (type(name).__name__=='list'):
      for each in name:
        self.components[each].show()
    else:
      self.components[each].show()

  # 隐藏组件
  def hide(self, name):
    if (type(name).__name__=='list'):
      for each in name:
        self.components[each].hide()
    else:
      self.components[each].hide()

  # 关闭组件
  def close(self, name):
    if (type(name).__name__=='list'):
      for each in name:
        self.components[each].close()
    else:
      self.components[each].close()

  # 移除组件
  def remove(self, name=None):
    if not name:
      for each in self.components.values():
        if hasattr(each, "destroy"):
          each.destroy()
        each.children.remove()  # 移除子组件的子组件
        self.components = {}
    else:
      if name in self.components.keys():
        each = self.components[name]
        each.children.remove() # 移除子组件的子组件
        if hasattr(each, "destroy"):
          each.destroy()        
        del each

  def __getitem__(self, __name):
    if __name in self.components.keys():
      return self.components[__name]
    else:
      print(self.components)
      return None
    
# 加载属性
def loadProps(self:object, props:any):
  if type(props) == str and os.path.exists(props):
    # 检查文件是被存在循环引用
    self.props = Param(filePath=props)
  elif type(props) == dict:
    self.props = Param(default=props)
  elif type(props) == Param:
    self.props = props
  else:
    self.props = Param()


import time, os
from nanoid import generate
from .Store import Store
from .Callback import Callback
from .Children import Children
from .Param import Param

class Page:
  def __init__(self, name=None):
    self.name = name
    self.id = generate(size=10)
    self.callback = Callback()                     # 挂载回调函数管理对象
    self.children = Children()                     # 挂载子页面管理对象
    self.store = Store()                           # 挂载全局变量管理对象
    self.props:Param
    self.events:Param
    self.onShow = None
    self.onHide = None

  # 页面初始化
  def setup(self, props=None):
    """
    1.加载和设置页面参数
    2.挂载子页面
    3.添加回调函数
    4.绑定事件
    """
    # 加载和设置页面参数
    loadProps(self, props)
    self.name = self.props.get('name', self.name)
    self.id = self.props.get('id', self.id)
    self.events = Param(default=self.props.get('events')) if self.props.has('events') else Param()
    # 挂载子页面
    self.children.setup(self.props.get('children'), createPage)
    # 添加回到函数
    addCallback(self, self.props.get('functions', {}))
    # 绑定事件
    executeBinds(self)
    # print(f'页面[{self.name}]{self.id}初始化完成 props:{self.props.get()}')
  
  def bind(self, signal:str, id:str, func:callable):
    self.events.set(f'{signal}/{id}', func)
    executeBind(self, signal, id, func)
    
  # 关闭页面
  def close(self):
    self.children.close()

  # 展示页面
  def show(self, *args):
    if type(self.onShow, callable):
      self.onShow(*args)

  # 隐藏页面
  def hide(self, *args):
    if type(self.onHide, callable):
      self.onHide(*args)

  def wait(self, second, callback):
    threadManager = self.store.get('threadManager', None)
    if not threadManager:
      return
    
    id = f"wait_{generate(size=6)}_{time.time()*1000}"
    def func(*args, **kwargs):
      try:
        callback(*args, **kwargs)
      except Exception as e:
        print("等待回调失败：", e)
      threadManager.remove(id)
      
    threadManager.add({
      "id": id,
      "callback": func,
      "function": lambda: time.sleep(second)
    }, True)

  def async_run(self, function, callback):
    threadManager = self.store.get('threadManager', None)
    if not threadManager:
      return
        
    id = f"async_run_{generate(size=6)}_{time.time()*1000}"
    def func(*args, **kwargs):
      try:
        callback(*args, **kwargs)
      except Exception as e:
        print("异步回调失败：", e)
      threadManager.remove(id)
    
    threadManager.add({
      "id": id,
      "callback": func,
      "function": function
    }, True)

  def call(self, name:str, *args):
    def run():
      self.callback.run(name, *args)
    return run

  # 查看组件信息
  @property
  def info(self):
    info_string = f"\n当前页面[{self.name}]有{len(self.children.components.keys())}子页面。\n"+self.children.info()
    return info_string

  def __getitem__(self, __name):
    return super().__getattribute__(__name)
  
  def __getattribute__(self, __name):
    if self["store"].has(__name):
      return self["store"].get(__name)
    else:
      return super().__getattribute__(__name)

# 创建页面
def createPage(name:str):
  return Page(name)

# 加载属性
def loadProps(self:Page, props:any):
  if type(props) == str and os.path.exists(props):
    self.props = Param(filePath=props)
  elif type(props) == dict:
    self.props = Param(default=props)
  elif type(props) == Param:
    self.props = props
  else:
    self.props = Param()

# 执行绑定列表
def executeBinds(page:Page):
  dataList = page.events.data
  for signal in dataList.keys():
    idDict = dataList[signal]
    if type(idDict) == dict:
      for id in idDict.keys():
        slot = page.events.data[signal][id]
        executeBind(page, signal, id, slot)

# 执行绑定
def executeBind(page:Page, signal:str, id:str, slot):
  functions = page.store.get('functions')
  if type(functions) == dict and '__signal_slots_func' in functions:
    __signal_slots_func = functions['__signal_slots_func']
    if hasattr(__signal_slots_func, '__call__'):
      if type(slot) == str:
        def call(name:str):
          return lambda *args: page.callback.run(name, *args)
      # 执行绑定函数
      __signal_slots_func(signal, id, call(slot))

# 添加回调函数      
def addCallback(page:Page, functions:dict):
  for name in functions.keys():
    def generate(pycode):
      if type(pycode) == str and os.path.exists(pycode):
        with open(pycode, 'r', encoding='utf-8') as f:
          pycode = f.read()
      elif type(pycode) == str:
        pass
      else:
        pycode = 'print("['+name+']未找到回调函数")'
      return lambda *args: exec(pycode, {"page":page, "args":args})
    # 添加到回调管理器
    page.callback.add(name, generate(functions[name]))


from collections import defaultdict

class Callback:
    def __init__(self):
        self.callback_dict = defaultdict(list)

    # 添加回调函数
    def add(self, name: str, function):
        self.callback_dict[name].append(function)

    # 获取回调函数
    def get(self, name):
        return self.callback_dict.get(name, [])

    # 运行回调函数
    def run(self, name, *args):
        result = []
        callbackList = self.get(name)
        for callback in callbackList:
            result.append(callback(*args) if callback else callback)
        return result

    # 移除回调函数
    def remove(self, name=None):
        if name is None:
            self.callback_dict.clear()
        elif name in self.callback_dict:
            del self.callback_dict[name]

    # 销毁回调管理对象
    def destroy(self):
        self.callback_dict.clear()